searchNodes=[{"doc":"Plug that handles Idempotency-Key HTTP headers. A single Idempotency-Key HTTP header is required for POST and PATCH requests. Handling of requests is based on https://datatracker.ietf.org/doc/draft-ietf-httpapi-idempotency-key-header/ Request ID The value of the Idempotency-Key HTTP header is combined with a URI to produce a unique ID for the request. This will be used to store the response for first-time requests. The ID is used to fetch this response in all subsequent requests. A fingerprint of the request payload is generated and used to ensure the ID is not reused with a different request payload. Error handling Concurrent requests will return a 409 Conflict response. Mismatch of request payload fingerprint will return 422 Unprocessable Entity response. If the first-time request was unexpectedly terminated a 500 Internal Server is returned. Cached responses and halted first-time requests, returning an Expires header in the response. See IdempotencyPlug.RequestTracker for more on expiration. Options :handler - the handler module to use for building the idempotent id and error response. Defaults to IdempotencyPlug.Handler . See IdempotencyPlug.Handler for more. :tracker - the name or pid for the IdempotencyPlug.RequestTracker GenServer. Defaults to IdempotencyPlug.RequestTracker . Examples plug IdempotencyPlug , tracker : IdempotencyPlug.RequestTracker , handler : IdempotencyPlug.Handler","ref":"IdempotencyPlug.html","title":"IdempotencyPlug","type":"module"},{"doc":"Module that defines an ETS store. The ETS table data is not persisted and will be flushed on application restart. You should use the IdempotencyPlug.EctoStore or a custom IdempotencyPlug.Store type module for production. Examples defmodule MyApp.Application do # .. def start ( _type , _args ) do children = [ { IdempotencyPlug.RequestTracker , [ store : { IdempotencyPlug.EctoStore , [ table : MyAppWeb.RequestTrackerStore ] } ] } # ... ] Supervisor . start_link ( children , strategy : :one_for_one , name : MyApp.Supervisor ) end end","ref":"IdempotencyPlug.ETSStore.html","title":"IdempotencyPlug.ETSStore","type":"module"},{"doc":"Module that defines an Ecto store. A migration file should be generated with mix idempotency_plug.ecto.gen.migration . Examples defmodule MyApp.Application do # .. def start ( _type , _args ) do children = [ { IdempotencyPlug.RequestTracker , [ store : { IdempotencyPlug.EctoStore , repo : MyApp.Repo } ] } # ... ] Supervisor . start_link ( children , strategy : :one_for_one , name : MyApp.Supervisor ) end end","ref":"IdempotencyPlug.EctoStore.html","title":"IdempotencyPlug.EctoStore","type":"module"},{"doc":"Module that defines the plug handler callbacks. Examples defmodule MyAppWeb.IdempotencyPlugHandler do @behaviour IdempotencyPlug.Handler import Phoenix.Controller import Plug.Conn @impl true def idempotent_id ( conn , id ) do IdempotencyPlug.Handler . idempotent_id ( conn , id ) end @impl true def resp_error ( conn , error ) do conn |&gt; put_status ( IdempotencyPlug.Handler . status ( error ) ) |&gt; json ( %{ error : IdempotencyPlug.Handler . message ( error ) } ) end end","ref":"IdempotencyPlug.Handler.html","title":"IdempotencyPlug.Handler","type":"behaviour"},{"doc":"Returns the request ID as-is.","ref":"IdempotencyPlug.Handler.html#idempotent_id/2","title":"IdempotencyPlug.Handler.idempotent_id/2","type":"function"},{"doc":"","ref":"IdempotencyPlug.Handler.html#c:idempotent_id/2","title":"IdempotencyPlug.Handler.idempotent_id/2","type":"callback"},{"doc":"Returns message for the error atom.","ref":"IdempotencyPlug.Handler.html#message/1","title":"IdempotencyPlug.Handler.message/1","type":"function"},{"doc":"Updates the conn with a JSON encoded response body and status code.","ref":"IdempotencyPlug.Handler.html#resp_error/2","title":"IdempotencyPlug.Handler.resp_error/2","type":"function"},{"doc":"","ref":"IdempotencyPlug.Handler.html#c:resp_error/2","title":"IdempotencyPlug.Handler.resp_error/2","type":"callback"},{"doc":"Returns the status for the error atom.","ref":"IdempotencyPlug.Handler.html#status/1","title":"IdempotencyPlug.Handler.status/1","type":"function"},{"doc":"","ref":"IdempotencyPlug.Handler.html#t:error/0","title":"IdempotencyPlug.Handler.error/0","type":"type"},{"doc":"","ref":"IdempotencyPlug.Handler.html#t:request_id/0","title":"IdempotencyPlug.Handler.request_id/0","type":"type"},{"doc":"A GenServer that tracks processes to ensure requests, at most, are processed once. Storage First-time tracked request will store {:processing, {node, pid}} request state with an expiration date for the provided request ID. Once the request has completed, put_response/3 must be called to update the cached response. The response will be stored as {:ok, data} with an expiration date. The process for a tracked request may halt unexpectedly (e.g. due to raised exception). This module will track the terminated process and store the value as {:halted, reason} . All cached responses will be removed after 24 hours. Lookup For subsequent requests, the state of the first-time tracked request will be returned in the format of {:cache, {:ok, data}, expires_at} . If the request payload fingerprint differs, {:mismatch, {:fingerprint, fingerprint}, expires_at} is returned. If first-time request has not yet completed, {:processing, {node, pid}, expires_at} is returned. If the request unexpectedly terminated, {:cache, {:halted, reason}, expires_at} is returned. Options :cache_ttl - the TTL in milliseconds for any objects in the cache store. Defaults to 24 hours. :prune - the interval in milliseconds to prune the cache store for expired objects. Defaults to 60 seconds. :store - the cache store module to use to store the cache objects. Defaults to {IdempotencyPlug.ETSStore, [table: Elixir.IdempotencyPlug.RequestTracker]} . Examples children = [ { IdempotencyPlug.RequestTracker , cache_ttl : :timer . hours ( 6 ) , prune : :timer . minutes ( 1 ) , store : { IdempotencyPlug.EctoStore , repo : MyApp.Repo } } ] Supervisor . start_link ( children , strategy : :one_for_one , name : MyApp.Supervisor )","ref":"IdempotencyPlug.RequestTracker.html","title":"IdempotencyPlug.RequestTracker","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"IdempotencyPlug.RequestTracker.html#child_spec/1","title":"IdempotencyPlug.RequestTracker.child_spec/1","type":"function"},{"doc":"Updates the state for a given request ID.","ref":"IdempotencyPlug.RequestTracker.html#put_response/3","title":"IdempotencyPlug.RequestTracker.put_response/3","type":"function"},{"doc":"","ref":"IdempotencyPlug.RequestTracker.html#start_link/1","title":"IdempotencyPlug.RequestTracker.start_link/1","type":"function"},{"doc":"Tracks a request ID. This function will return {:init, id, expires_at} for first-time requests. Subsequent requests will return the request state. If the request payload fingerprint differs from what was stored, an error is returned.","ref":"IdempotencyPlug.RequestTracker.html#track/3","title":"IdempotencyPlug.RequestTracker.track/3","type":"function"},{"doc":"Module that defines the store callbacks. Examples defmodule CustomStore do @behaviour IdempotencyPlug.Store @impl true def setup ( options ) @impl true def lookup ( request_id , options ) @impl true def insert ( request_id , fingerprint , data , options ) @impl true def update ( request_id , data , options ) @impl true def prune ( options ) end","ref":"IdempotencyPlug.Store.html","title":"IdempotencyPlug.Store","type":"behaviour"},{"doc":"","ref":"IdempotencyPlug.Store.html#c:insert/5","title":"IdempotencyPlug.Store.insert/5","type":"callback"},{"doc":"","ref":"IdempotencyPlug.Store.html#c:lookup/2","title":"IdempotencyPlug.Store.lookup/2","type":"callback"},{"doc":"","ref":"IdempotencyPlug.Store.html#c:prune/1","title":"IdempotencyPlug.Store.prune/1","type":"callback"},{"doc":"","ref":"IdempotencyPlug.Store.html#c:setup/1","title":"IdempotencyPlug.Store.setup/1","type":"callback"},{"doc":"","ref":"IdempotencyPlug.Store.html#c:update/4","title":"IdempotencyPlug.Store.update/4","type":"callback"},{"doc":"","ref":"IdempotencyPlug.Store.html#t:data/0","title":"IdempotencyPlug.Store.data/0","type":"type"},{"doc":"","ref":"IdempotencyPlug.Store.html#t:expires_at/0","title":"IdempotencyPlug.Store.expires_at/0","type":"type"},{"doc":"","ref":"IdempotencyPlug.Store.html#t:fingerprint/0","title":"IdempotencyPlug.Store.fingerprint/0","type":"type"},{"doc":"","ref":"IdempotencyPlug.Store.html#t:options/0","title":"IdempotencyPlug.Store.options/0","type":"type"},{"doc":"","ref":"IdempotencyPlug.Store.html#t:request_id/0","title":"IdempotencyPlug.Store.request_id/0","type":"type"},{"doc":"Generates a IdempotencyPlug store migration. See Mix.Tasks.Ecto.Gen.Migration for options, takes all options except --change .","ref":"Mix.Tasks.IdempotencyPlug.Ecto.Gen.Migration.html","title":"Mix.Tasks.IdempotencyPlug.Ecto.Gen.Migration","type":"task"},{"doc":"Plug that makes POST and PATCH requests idempotent using Idempotency-Key HTTP header. Follows the IETF Idempotency-Key HTTP Header Field specification draft .","ref":"README.html","title":"IdempotencyPlug","type":"extras"},{"doc":"Add idempotency_plug to your list of dependencies in mix.exs : def deps do [ { :idempotency_plug , &quot;~&gt; 0.1&quot; } ] end","ref":"README.html#installation","title":"IdempotencyPlug - Installation","type":"extras"},{"doc":"First add the request tracker to your supervision tree: defmodule MyApp.Application do # .. def start ( _type , _args ) do children = [ { IdempotencyPlug.RequestTracker , [ name : MyAppWeb.RequestTracker ] } # ... ] Supervisor . start_link ( children , strategy : :one_for_one , name : MyApp.Supervisor ) end end Now add the plug to your pipeline: defmodule MyAppWeb.Router do # ... pipeline :api do plug :accepts , [ &quot;json&quot; ] plug IdempotencyPlug , tracker : MyAppWeb.RequestTracker end # ... end All POST and PATCH requests will now be idempotent using the Idempotency-Key HTTP header value, storing responses with the default ETS store. Persisted store The ETS store is not persisted, so it's not production ready. Instead, let's change the store to use Ecto. First, run mix idempotency_plug.ecto.gen.migration . Now update the configuration for the request tracker: { IdempotencyPlug.RequestTracker , [ store : { IdempotencyPlug.EctoStore , repo : MyApp.Repo } ] } You can also implement your own idempotent request store by using the behaviour in IdempotencyPlug.Store .","ref":"README.html#usage","title":"IdempotencyPlug - Usage","type":"extras"},{"doc":"The plug handling can be customized by using the IdempotencyPlug.Handler behaviour: defmodule MyAppWeb.IdempotencyPlugHandler do @behaviour IdempotencyPlug.Handler import Phoenix.Controller import Plug.Conn @impl true def idempotent_id ( conn , id ) do IdempotencyPlug.Handler . idempotent_id ( conn , id ) end @impl true def resp_error ( conn , error ) do conn |&gt; put_status ( IdempotencyPlug.Handler . status ( error ) ) |&gt; json ( %{ error : IdempotencyPlug.Handler . message ( error ) } ) end end Remember to update the plug opts: plug IdempotencyPlug , tracker : MyAppWeb.RequestTracker , handler : MyAppWeb.IdempotencyPlugHandler","ref":"README.html#customize-response","title":"IdempotencyPlug - Customize response","type":"extras"},{"doc":"If you authenticate a user in your API you will need to scope the Idempotency-Key to the authenticated user: defmodule MyAppWeb.IdempotencyPlugHandler do @behaviour IdempotencyPlug.Handler @impl true def idempotent_id ( conn , id ) do &quot; \#{ conn . assigns . current_user . id } : \#{ id } &quot; end @impl true def resp_error ( conn , error ) do IdempotencyPlug.Handler . resp_error ( conn , error ) end end","ref":"README.html#scope-idempotency-key","title":"IdempotencyPlug - Scope Idempotency-Key","type":"extras"},{"doc":"For your controller tests you may want to add this helper to set up the idempotency key: def setup_with_idempotency_key ( %{ conn : conn } ) do conn = Plug.Conn . put_req_header ( conn , &quot;idempotency-key&quot; , Ecto.UUID . bingenerate ( ) ) { :ok , conn : conn } end setup :setup_with_idempotency_key This ensures that all the tests succeed by generating a UUID for all requests.","ref":"README.html#phoenix-tests","title":"IdempotencyPlug - Phoenix tests","type":"extras"},{"doc":"(The MIT License) Copyright (c) 2023 Dan Schultzer &amp; the Contributors Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the 'Software'), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions: The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"README.html#license","title":"IdempotencyPlug - LICENSE","type":"extras"},{"doc":"","ref":"CHANGELOG.html","title":"CHANGELOG","type":"extras"},{"doc":"Fix source url and name in docs","ref":"CHANGELOG.html#v0-1-2-2023-04-07","title":"CHANGELOG - v0.1.2 (2023-04-07)","type":"extras"},{"doc":"Fix indention for generated Ecto migration","ref":"CHANGELOG.html#v0-1-1-2023-04-07","title":"CHANGELOG - v0.1.1 (2023-04-07)","type":"extras"},{"doc":"Initial release","ref":"CHANGELOG.html#v0-1-0-2023-04-06","title":"CHANGELOG - v0.1.0 (2023-04-06)","type":"extras"}]